@@ -88,14 +88,17 @@ welcome_message_post_setup (void)
 }
 
 static void
-print_tag_list (const char **tags, size_t tags_len)
+print_tag_list (notmuch_config_values_t *tags)
 {
-    unsigned int i;
+    bool first = false;
 
-    for (i = 0; i < tags_len; i++) {
-	if (i != 0)
+    for (;
+	 notmuch_config_values_valid (tags);
+	 notmuch_config_values_move_to_next (tags)) {
+	if (! first)
 	    printf (" ");
-	printf ("%s", tags[i]);
+	first = false;
+	printf ("%s", notmuch_config_values_get (tags));
     }
 }
 
@@ -121,19 +124,14 @@ parse_tag_list (void *ctx, char *response)
 }
 
 int
-notmuch_setup_command (notmuch_config_t *config,
+notmuch_setup_command (notmuch_database_t *notmuch,
 		       int argc, char *argv[])
 {
     char *response = NULL;
     size_t response_size = 0;
-    const char **old_other_emails;
-    size_t old_other_emails_len;
     GPtrArray *other_emails;
-    unsigned int i;
-    const char **new_tags;
-    size_t new_tags_len;
-    const char **search_exclude_tags;
-    size_t search_exclude_tags_len;
+    notmuch_config_values_t *new_tags, *search_exclude_tags, *emails;
+    notmuch_conffile_t *config;
 
 #define prompt(format, ...)                                     \
     do {                                                        \
@@ -153,29 +151,35 @@ notmuch_setup_command (notmuch_config_t *config,
 	fprintf (stderr, "Warning: ignoring --uuid=%s\n",
 		 notmuch_requested_db_uuid);
 
-    if (notmuch_config_is_new (config))
+    config = notmuch_conffile_open (notmuch,
+				    notmuch_config_path (notmuch), true);
+    if (! config)
+	return EXIT_FAILURE;
+
+    if (notmuch_conffile_is_new (config))
 	welcome_message_pre_setup ();
 
-    prompt ("Your full name [%s]: ", notmuch_config_get_user_name (config));
+    prompt ("Your full name [%s]: ", notmuch_config_get (notmuch, NOTMUCH_CONFIG_USER_NAME));
     if (strlen (response))
-	notmuch_config_set_user_name (config, response);
+	notmuch_conffile_set_user_name (config, response);
 
     prompt ("Your primary email address [%s]: ",
-	    notmuch_config_get_user_primary_email (config));
+	    notmuch_config_get (notmuch, NOTMUCH_CONFIG_PRIMARY_EMAIL));
     if (strlen (response))
-	notmuch_config_set_user_primary_email (config, response);
+	notmuch_conffile_set_user_primary_email (config, response);
 
     other_emails = g_ptr_array_new ();
 
-    old_other_emails = notmuch_config_get_user_other_email (config,
-							    &old_other_emails_len);
-    for (i = 0; i < old_other_emails_len; i++) {
-	prompt ("Additional email address [%s]: ", old_other_emails[i]);
+    for (emails = notmuch_config_get_values (notmuch, NOTMUCH_CONFIG_OTHER_EMAIL);
+	 notmuch_config_values_valid (emails);
+	 notmuch_config_values_move_to_next (emails)) {
+	const char *email = notmuch_config_values_get (emails);
+
+	prompt ("Additional email address [%s]: ", email);
 	if (strlen (response))
 	    g_ptr_array_add (other_emails, talloc_strdup (config, response));
 	else
-	    g_ptr_array_add (other_emails, talloc_strdup (config,
-							  old_other_emails[i]));
+	    g_ptr_array_add (other_emails, talloc_strdup (config, email));
     }
 
     do {
@@ -184,57 +188,59 @@ notmuch_setup_command (notmuch_config_t *config,
 	    g_ptr_array_add (other_emails, talloc_strdup (config, response));
     } while (strlen (response));
     if (other_emails->len)
-	notmuch_config_set_user_other_email (config,
-					     (const char **)
-					     other_emails->pdata,
-					     other_emails->len);
+	notmuch_conffile_set_user_other_email (config,
+					       (const char **)
+					       other_emails->pdata,
+					       other_emails->len);
     g_ptr_array_free (other_emails, true);
 
     prompt ("Top-level directory of your email archive [%s]: ",
-	    notmuch_config_get_database_path (config));
+	    notmuch_config_get (notmuch, NOTMUCH_CONFIG_DATABASE_PATH));
     if (strlen (response)) {
 	const char *absolute_path;
 
 	absolute_path = make_path_absolute (config, response);
-	notmuch_config_set_database_path (config, absolute_path);
+	notmuch_conffile_set_database_path (config, absolute_path);
     }
 
-    new_tags = notmuch_config_get_new_tags (config, &new_tags_len);
+    new_tags = notmuch_config_get_values (notmuch, NOTMUCH_CONFIG_NEW_TAGS);
 
     printf ("Tags to apply to all new messages (separated by spaces) [");
-    print_tag_list (new_tags, new_tags_len);
+    print_tag_list (new_tags);
     prompt ("]: ");
 
     if (strlen (response)) {
 	GPtrArray *tags = parse_tag_list (config, response);
 
-	notmuch_config_set_new_tags (config, (const char **) tags->pdata,
-				     tags->len);
+	notmuch_conffile_set_new_tags (config, (const char **) tags->pdata,
+				       tags->len);
 
 	g_ptr_array_free (tags, true);
     }
 
-
-    search_exclude_tags = notmuch_config_get_search_exclude_tags (config, &search_exclude_tags_len);
+    search_exclude_tags = notmuch_config_get_values (notmuch, NOTMUCH_CONFIG_EXCLUDE_TAGS);
 
     printf ("Tags to exclude when searching messages (separated by spaces) [");
-    print_tag_list (search_exclude_tags, search_exclude_tags_len);
+    print_tag_list (search_exclude_tags);
     prompt ("]: ");
 
     if (strlen (response)) {
 	GPtrArray *tags = parse_tag_list (config, response);
 
-	notmuch_config_set_search_exclude_tags (config,
-						(const char **) tags->pdata,
-						tags->len);
+	notmuch_conffile_set_search_exclude_tags (config,
+						  (const char **) tags->pdata,
+						  tags->len);
 
 	g_ptr_array_free (tags, true);
     }
 
-    if (notmuch_config_save (config))
+    if (notmuch_conffile_save (config))
 	return EXIT_FAILURE;
 
-    if (notmuch_config_is_new (config))
+    if (config)
+	notmuch_conffile_close (config);
+
+    if (notmuch_conffile_is_new (config))
 	welcome_message_post_setup ();
 
     return EXIT_SUCCESS;
